package net.pieroxy.tools.j2ee;
import javax.servlet.http.*;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import java.io.IOException;
import java.util.regex.*;
import java.util.*;

/**
 * RedirectFilter is a class that allows for simple static 
 * redirections in a web app. The configuration is stored in the 
 * web.xml file. For more information, visit the website at 
 * <a target="_top" href="http://pieroxy.net/java/redirectfilter/">http://pieroxy.net/java/redirectfilter/</a>
 * @author pieroxy
 *
 */
public class RedirectFilter implements Filter {
  Map<Pattern,String> matchersRE = new HashMap<Pattern,String>();
  Map<String,String> matchersSS = new HashMap<String,String>();

  private int configStatus = 301;
  private boolean configAddQueryString = false;
  boolean matchRegexp = true;
  int verboseLevel = 0;

  /**
   * Initializes the filter.
   * 
   * @param arg0 The filter configuration generated by the servlet container.
   */
  public void init(FilterConfig arg0) throws ServletException {
    Enumeration e = arg0.getInitParameterNames();
    while (true) {
      if (!e.hasMoreElements()) break;
      String name = (String)e.nextElement();
      String value = arg0.getInitParameter(name).trim();
      if (name.startsWith("RedirectFilterConfig:")) {
        name = name.substring(21);
        if (name.equals("status")) {
          try {
            configStatus = Integer.parseInt(value);
          } catch (Exception ex) {
            System.err.println("Unable to parse status from init parameters: " + value);
          }
        } else
          if (name.equals("addQueryString")) {
            try {
              configAddQueryString = Boolean.parseBoolean(value);
            } catch (Exception ex) {
              System.err.println("Unable to parse query string parameter from init parameters: " + value);
            }
          } else 
            if (name.equals("patternType")) {
              if (value.equals("regexp")) {
                matchRegexp=true;
              } else if (value.equals("substring")) {
                matchRegexp=false;
              } else {
                System.err.println("RedirectFilter: Unknown config parameter for patternType: " + value);
              }
            } else if (name.equals("verboseLevel")) {
              try {
                verboseLevel = Integer.parseInt(value);
              } catch (Exception ex) {
                System.err.println("Unable to parse verbose level parameter from init parameters: " + value);
              }
            } else {
              System.err.println("RedirectFilter: Unknown config parameter name: " + name);
            }
      }
    }

    e = arg0.getInitParameterNames();
    while (true) {
      if (!e.hasMoreElements()) break;
      String name = (String)e.nextElement();
      String value = arg0.getInitParameter(name).trim();
      if (!name.startsWith("RedirectFilterConfig:")) {
        if (matchRegexp) {
          matchersRE.put(Pattern.compile(name),value);
        } else {
          matchersSS.put(name,value);
        }
      }
    }
  }

  /**
   * Does the actual job of checking for possible redirections
   * and to redirect the request in case a match is found.
   * 
   * @param chain The filter chain. This is generated by the servlet container.
   * @param servletRequest The request. This is generated by the servlet container.
   * @param servletResponse The response. This is generated by the servlet container.
   */
  public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain chain) throws IOException, ServletException {
    HttpServletRequest request = (HttpServletRequest)servletRequest;
    HttpServletResponse response = (HttpServletResponse)servletResponse;
    String u = request.getRequestURL().toString();
    String rt = checkURL(u);
    if (rt==null) {
      if (verboseLevel>1)
        System.out.println(new Date() + " RedirectFilter::No redirection for " + u);
      chain.doFilter(request,response);
    } else {
      response.setStatus(configStatus);
      if (configStatus == 301 || configStatus == 302) {
        if (configAddQueryString) {
          String qs = request.getQueryString();
          if (qs!=null && qs.length()>0) {
            rt += "?" + qs;
          }
        }
        response.addHeader("Location",rt);
        if (verboseLevel>0)
          System.out.println(new Date() + " RedirectFilter::Returning http status " + configStatus + " for " + u + " with new location as " + rt);
      } else {
        if (verboseLevel>0)
          System.out.println(new Date() + " RedirectFilter::Returning http status " + configStatus + " for " + u);
      }
    }		
  }

  /**
   * Does nothing.
   */
  public void destroy() {
  }




  private String checkURL(String URL) {
    if (matchRegexp) return checkURL_RE(URL);
    else return checkURL_SS(URL);
  }
  private String checkURL_RE(String URL) {
    Iterator e = matchersRE.keySet().iterator();
    while (true) {
      if (!e.hasNext()) break;
      Pattern pattern = (Pattern)e.next();
      Matcher match = pattern.matcher(URL);
      if (match.find()) {
        return match.replaceAll(matchersRE.get(pattern));
      }
    }
    return null;
  }
  private String checkURL_SS(String URL) {
    Iterator e = matchersSS.keySet().iterator();
    while (true) {
      if (!e.hasNext()) break;
      String pattern = (String)e.next();
      int pos = URL.indexOf(pattern);
      if (pos>-1) {
        return URL.replace(pattern,matchersSS.get(pattern));
      }
    }
    return null;
  }

  public boolean isConfigAddQueryString() {
    return configAddQueryString;
  }

  public int getConfigStatus() {
    return configStatus;
  }

  public boolean isMatchRegexp() {
    return matchRegexp;
  }

  public int getVerboseLevel() {
    return verboseLevel;
  }


}
